/*============================================================================
  File:     IndexMissingStats.sql

  Summary:  Be careful... don't just add all of the "missing" indexes! 
			But, this should give you a comprehensive look at what 
			indexes are being suggested by queries that have run. In addition
			to this you might want to check out the DMVPlanCache script to
			see which of the most expensive queries have run - then start
			to analyze those and combine the recommendations (possibly from
			using DTA) to see if the expensive and most-frequently executed 
			queries are actually needing these missing/recommended indexes.
           
  Date:     June 2008

  SQL Server Version: 9.00.3054.00 (SP2)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SELECT g.*
FROM sys.dm_db_missing_index_groups AS g
go

SELECT s.*
FROM sys.dm_db_missing_index_group_stats AS s
go

SELECT d.*
FROM sys.dm_db_missing_index_details AS d
go

SELECT d.*
		, s.avg_total_user_cost
		, s.avg_user_impact
		, s.last_user_seek
		, s.unique_compiles
		, s.*
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
ORDER BY s.avg_user_impact DESC
go

-- Here's a quick query of the index statements that this workload (with all things
-- being equal) would create:

SELECT distinct *
FROM 
((SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.equality_columns + N') INCLUDE (' + d.included_columns + N' )' AS IndexStatement
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NOT NULL 
	AND d.inequality_columns IS NULL
	AND d.included_columns IS NOT NULL)
UNION
(SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.inequality_columns + N') INCLUDE (' + d.included_columns + N' )' 
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NULL 
	AND d.inequality_columns IS NOT NULL
	AND d.included_columns IS NOT NULL)
UNION
(SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.equality_columns + N') '
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NOT NULL 
	AND d.inequality_columns IS NULL
	AND d.included_columns IS NULL)
UNION
(SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.inequality_columns + N') '
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NULL 
	AND d.inequality_columns IS NOT NULL
	AND d.included_columns IS NULL)
UNION
(SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.equality_columns + ', ' + d.inequality_columns + N') INCLUDE (' + d.included_columns + N' )' 
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NOT NULL AND d.inequality_columns IS NOT NULL
	AND d.included_columns IS NOT NULL)
UNION
(SELECT 	N'CREATE INDEX <<IndName>> ON ' + d.statement 
		+ N' (' + d.equality_columns + ', ' + d.inequality_columns + N') '
FROM sys.dm_db_missing_index_group_stats AS s
	JOIN sys.dm_db_missing_index_groups AS g
		ON s.group_handle = g.index_group_handle 
	JOIN sys.dm_db_missing_index_details AS d
		ON d.index_handle = g.index_handle
WHERE d.equality_columns IS NOT NULL AND d.inequality_columns IS NOT NULL
	AND d.included_columns IS NULL)) AS MissingIndexes
go